/********************************************************
 * Data Visualization - 2D Matrix Demo                  *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author: Rodger (Jieliang) Luo 
 * Re-edit by Weidi Zhang
 * Supervisor: George Legrady                           *
 *                                                      *
 * Purpose: Demonstrate how to make a simple 2D Matrix  *
 *          data visualization with different color     *
 *          schemes.                                    *
 *                                                      *
 ********************************************************/

/*
Step One: Upload CSV to Table & Get basic information of data. 
 Step Two: Write data from Table to 2D Array & Have max and min value of data.
 Step Three: Visualize 2D Matrix.
 Step Four: Draw labels and title.
 */
Table table;
int numCols, numRows;
int [][]dataMatrix;
int dataMax, dataMin;
float cellH, cellW;
float marginTop = 20, marginLeft = 50;
int mapped;
int startYear = 2006, endYear = 2017;

void setup() {
  size(800, 800);
  surface.setResizable(true);
  table = loadTable("item_month.csv", "header");
  //get the numbers of rows and columns
  numCols = table.getColumnCount();
  numRows = table.getRowCount();
  println("the total row is " + numRows +" the total columns is " + numCols);

  // step2 ...
  dataMatrix = new int[numRows][numCols];
  for (int i=0; i<numRows; i++) {
    for (int j=0; j<numCols-3; j++) {
      dataMatrix[i][j] = table.getInt(i, j+3);
    }
  }
  // step2 find max and min
  for (int i =0; i< numRows; i++) {
    for (int j=0; j<numCols-3; j++) {
      if (dataMatrix[i][j]>dataMax) {
        dataMax = dataMatrix[i][j];
      }
    }
  }
  dataMin = dataMax;
  for (int i =0; i< numRows; i++) {
    for (int j=0; j<numCols-3; j++) {
      if (dataMatrix[i][j]<dataMin) {
        dataMin = dataMatrix[i][j];
      }
    }
  }
  println("dataMax is " + dataMax + " dataMin is " + dataMin);
}

void draw() {
  background(255, 255, 255);
  drawTable();
  drawLabels();
  userInteraction();
}

void drawTable() {
  cellW = (width - marginLeft*2)/(numCols-3);
  cellH = (height - marginTop*2)/numRows;

  //map transparency to the value of data
  for (int i=0; i<numRows; i++) {
    for (int j=0; j<numCols-3; j++) {
      mapped = int(map(dataMatrix[i][j+3], dataMin, dataMax, 20, 255));

      //2479267 
      //1943806
      //2132112
      //3284034
      if (table.getInt(i, 0)==2479267) {
        fill(255, 0, 0, mapped);
      } else if (table.getInt(i, 0)==1943806) {
        fill(0, 255, 140, mapped);
      } else if (table.getInt(i, 0)==2132112) {
        fill(0, 120, 255, mapped);
      } else if (table.getInt(i, 0)==3284034) {
        fill(180, 100, 0, mapped);
      }

      stroke(10, 10, 10, 20);
      rect(marginLeft+cellW*j, marginTop+cellH*i, cellW, cellH);
    }
  }
}

void drawLabels() {
  //draw timeline
  for ( int i =0; i<(endYear-startYear+1); i++) {
    fill(0);
    stroke(0);
    textSize(12);
    textAlign(CENTER);
    text(startYear+i, marginLeft+cellW*(i*12+6), height-marginTop/2);
    stroke(255, 0, 0, 100);
    // line(x1, y1, x2,y2);
    line(marginLeft+cellW*i*12, height-marginTop, marginLeft+cellW*i*12, height-marginTop/2);
  }

  //drawLabels
  for (int i=0; i<numRows; i++) {
    textSize(5);
    fill(0, 0, 0, 180);
    textAlign(CENTER, TOP);
    text(table.getString(i, 0) +" " +table.getString(i, 1), marginLeft/2, marginTop+i*cellH);
  }

  //draw title
  textSize(10);
  textAlign(CENTER, CENTER);
  text ("Blade Runner Monthly Performance from 2006 to 2017", width/2, marginTop/2);
}
